@@ -242,20 +242,6 @@ class WP_REST_Server {
 		$this->send_header( 'Access-Control-Expose-Headers', 'X-WP-Total, X-WP-TotalPages' );
 		$this->send_header( 'Access-Control-Allow-Headers', 'Authorization, Content-Type' );
 
-		/**
-		 * Send nocache headers on authenticated requests.
-		 *
-		 * @since 4.4.0
-		 *
-		 * @param bool $rest_send_nocache_headers Whether to send no-cache headers.
-		 */
-		$send_no_cache_headers = apply_filters( 'rest_send_nocache_headers', is_user_logged_in() );
-		if ( $send_no_cache_headers ) {
-			foreach ( wp_get_nocache_headers() as $header => $header_value ) {
-				$this->send_header( $header, $header_value );
-			}
-		}
-
 		/**
 		 * Filters whether the REST API is enabled.
 		 *
@@ -311,10 +297,12 @@ class WP_REST_Server {
 		 * $_GET['_method']. If that is not set, we check for the HTTP_X_HTTP_METHOD_OVERRIDE
 		 * header.
 		 */
+		$method_overridden = false;
 		if ( isset( $_GET['_method'] ) ) {
 			$request->set_method( $_GET['_method'] );
 		} elseif ( isset( $_SERVER['HTTP_X_HTTP_METHOD_OVERRIDE'] ) ) {
 			$request->set_method( $_SERVER['HTTP_X_HTTP_METHOD_OVERRIDE'] );
+			$method_overridden = true;
 		}
 
 		$result = $this->check_authentication();
@@ -373,6 +361,24 @@ class WP_REST_Server {
 		 */
 		$served = apply_filters( 'rest_pre_serve_request', false, $result, $request, $this );
 
+		/**
+		 * Filters whether to send nocache headers on a REST API request.
+		 *
+		 * @since 4.4.0
+		 * @since 6.x.x Moved the block to catch the filter added on rest_cookie_check_errors() from rest-api.php
+		 *
+		 * @param bool $rest_send_nocache_headers Whether to send no-cache headers.
+		 */
+		$send_no_cache_headers = apply_filters( 'rest_send_nocache_headers', is_user_logged_in() );
+
+		// send no cache headers if the $send_no_cache_headers is true
+		// OR if the HTTP_X_HTTP_METHOD_OVERRIDE is used but resulted a 4xx response code.
+		if ( $send_no_cache_headers || ( true === $method_overridden && strpos( $code, '4' ) === 0 ) ) {
+			foreach ( wp_get_nocache_headers() as $header => $header_value ) {
+				$this->send_header( $header, $header_value );
+			}
+		}
+
 		if ( ! $served ) {
 			if ( 'HEAD' === $request->get_method() ) {
 				return null;
@@ -1010,13 +1016,15 @@ class WP_REST_Server {
 	public function get_index( $request ) {
 		// General site data.
 		$available = array(
-			'name'           => get_option( 'blogname' ),
-			'description'    => get_option( 'blogdescription' ),
-			'url'            => get_option( 'siteurl' ),
-			'home'           => home_url(),
-			'namespaces'     => array_keys( $this->namespaces ),
-			'authentication' => array(),
-			'routes'         => $this->get_data_for_routes( $this->get_routes(), $request['context'] ),
+			'name'            => get_option( 'blogname' ),
+			'description'     => get_option( 'blogdescription' ),
+			'url'             => get_option( 'siteurl' ),
+			'home'            => home_url(),
+			'gmt_offset'      => get_option( 'gmt_offset' ),
+			'timezone_string' => get_option( 'timezone_string' ),
+			'namespaces'      => array_keys( $this->namespaces ),
+			'authentication'  => array(),
+			'routes'          => $this->get_data_for_routes( $this->get_routes(), $request['context'] ),
 		);
 
 		$response = new WP_REST_Response( $available );
